@@ -111,7 +111,9 @@ main(int argc, char *argv[])
     ctl_timeout_setup(timeout);
 
     /* Initialize IDL. */
-    idl = the_idl = ovsdb_idl_create(db, &icsbrec_idl_class, true, false);
+    idl = the_idl = ovsdb_idl_create_unconnected(&icsbrec_idl_class, true);
+    ovsdb_idl_set_remote(idl, db, false);
+    ovsdb_idl_set_db_change_aware(idl, false);
     ovsdb_idl_set_leader_only(idl, leader_only);
     run_prerequisites(commands, n_commands, idl);
 
@@ -747,9 +749,9 @@ static const struct ctl_table_class tables[ICSBREC_N_TABLES] = {
 
 static void
 ic_sbctl_context_init_command(struct ic_sbctl_context *ic_sbctl_ctx,
-                           struct ctl_command *command)
+                           struct ctl_command *command, bool last_command)
 {
-    ctl_context_init_command(&ic_sbctl_ctx->base, command);
+    ctl_context_init_command(&ic_sbctl_ctx->base, command, last_command);
 }
 
 static void
@@ -833,7 +835,8 @@ do_ic_sbctl(const char *args, struct ctl_command *commands, size_t n_commands,
     }
     ic_sbctl_context_init(&ic_sbctl_ctx, NULL, idl, txn, symtab);
     for (c = commands; c < &commands[n_commands]; c++) {
-        ic_sbctl_context_init_command(&ic_sbctl_ctx, c);
+        ic_sbctl_context_init_command(&ic_sbctl_ctx, c,
+                                      c == &commands[n_commands - 1]);
         if (c->syntax->run) {
             (c->syntax->run)(&ic_sbctl_ctx.base);
         }
